#include <Arduino.h>
#include <pico.h>
#include <hardware/pwm.h>
#include <Keyboard.h>
#define PORT 2
#define PORT2 3
class HallButton
{
public:
  uint64_t firstPulseStart;
  uint64_t firstPulseLength;
  bool isDataReadyToRead;

  HallButton(uint8_t gpioNum, char buttonOnKeyboard)
  {
    this->gpioNum = gpioNum;
    this->buttonOnKeyboard = buttonOnKeyboard;
  }

  void HandleISR(bool gpioState, uint64_t currentTime)
  {
    if (gpioState)
    {
      firstPulseLength = currentTime - firstPulseStart;
      isDataReadyToRead = true;
    }
    else
    {
      firstPulseStart = currentTime;
    }
  }
  void HandleButton()
  {
    if (!buttonPressed && firstPulseLength > 310)
    {
      Keyboard.press(buttonOnKeyboard);
      buttonPressed = true;
    }
    else if (buttonPressed && firstPulseLength < 300)
    {
      Keyboard.release(buttonOnKeyboard);
      buttonPressed = false;
    }
  }

private:
  bool buttonPressed;
  uint8_t gpioNum;
  char buttonOnKeyboard;
};
HallButton btn[2] = {HallButton(PORT, 't'), HallButton(PORT2, 'y')};
void isr(uint gpio, uint32_t events)
{
  uint64_t currentTime = time_us_64();
  bool state = events == 8;
  switch (gpio)
  {
  case PORT:
    btn[0].HandleISR(state, currentTime);
    break;
  case PORT2:
    btn[1].HandleISR(state, currentTime);
    break;
  }
}

void setup()
{
  Keyboard.begin();
  Serial.begin(115200);
  gpio_init(PORT);
  gpio_set_dir(PORT, false);
  gpio_pull_up(PORT);
  gpio_set_irq_enabled_with_callback(PORT, GPIO_IRQ_EDGE_RISE | GPIO_IRQ_EDGE_FALL, true, isr);
  gpio_init(PORT2);
  gpio_set_dir(PORT2, false);
  gpio_pull_up(PORT2);
  gpio_set_irq_enabled_with_callback(PORT2, GPIO_IRQ_EDGE_RISE | GPIO_IRQ_EDGE_FALL, true, isr);
}
#define printf Serial.printf
byte counter = 0;
void printDebugInfo()
{
  if (counter < 1 && btn[0].isDataReadyToRead)
  {
    btn[0].isDataReadyToRead = false;
    printf("BTN1: %lu", btn[0].firstPulseLength);
    counter++;
  }
  if (btn[1].isDataReadyToRead)
  {
    btn[1].isDataReadyToRead = false;
    if (counter > 0)
      printf("\t");
    printf("BTN2: %lu", btn[1].firstPulseLength);
    counter++;
  }
  if (counter > 1)
  {
    printf("\n");
    counter = 0;
  }
}
void loop()
{
  btn[0].HandleButton();
  btn[1].HandleButton();
}